@@ -1,7 +1,7 @@
 package Mojar::Mysql;
 use Mojo::Base -strict;
 
-our $VERSION = 2.063;
+our $VERSION = 2.072;
 
 1;
 __END__
@@ -12,4 +12,5 @@ Mojar::Mysql - Suite of MySQL integration tools.
 
 =head1 DESCRIPTION
 
-This initial release contains the database connector L<Mojar::Mysql::Connector>.
+This release includes the database connector L<Mojar::Mysql::Connector>, and
+the utility pack L<Mojar::Mysql::Util>.
